/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.chesspdfbrowser.application;

import com.frojasg1.applications.common.configuration.application.BaseApplicationConfigurationInterface;
import com.frojasg1.chesspdfbrowser.configuration.ApplicationConfiguration;
import com.frojasg1.general.desktop.view.pdf.ImageJPanel;
import com.frojasg1.general.desktop.view.splash.GenericBasicSplash;
import com.hotmail.frojasg1.applications.common.components.resizecomp.MapResizeRelocateComponentItem;
import com.hotmail.frojasg1.applications.common.components.resizecomp.ResizeRelocateItem;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.util.Vector;
import javax.swing.JPopupMenu;

/**
 *
 * @author Usuario
 */
public class Splash extends GenericBasicSplash
{
	protected static Splash _instance = null;
	public static final String sa_configurationBaseFileName = "Splash";

	public static final String CONF_OPENING_CONFIGURATION = "OPENING_CONFIGURATION";
	public static final String CONF_SHOWING_LICENSES = "SHOWING_LICENSES";
	public static final String CONF_IMPORTING_CONFIGURATION = "IMPORTING_CONFIGURATION";
	public static final String CONF_CREATING_MAIN_WINDOW = "CREATING_MAIN_WINDOW";
	public static final String CONF_PROBLEM_LAUNCHING_THE_APPLICATION = "PROBLEM_LAUNCHING_THE_APPLICATION";

	public static Splash instance()
	{
		if( _instance == null )
			_instance = new Splash( ApplicationConfiguration.instance() );

		return( _instance );
	}

	/**
	 * Creates new form Splash
	 */
	protected Splash( BaseApplicationConfigurationInterface applicationConfiguration )
	{
		super(applicationConfiguration);
//		initComponents();

		initOwnComponents();
		setCenteredLocation();
//		activateProgress(false );

		setListenersRecursive( this );
		
		setWindowConfiguration( );
	}

	protected void initOwnComponents()
	{
		setImage( getImage( AboutJDialog.RESOURCE_FOR_ABOUT_ICON ) );
		
		Dimension imageDimensions = getSizeOfImageJPanel();
		int xx = (int) (imageDimensions.getWidth() / 2);
		int yyIncrement = (int) (imageDimensions.getHeight() / 6);

		Font font = new Font("Calibri", Font.BOLD, 20);
		setFontForTextLines( font );

		addTextLine( new ImageJPanel.TextLine( "Chess PDF", new Point( xx, 2 * yyIncrement ) ) );
		addTextLine( new ImageJPanel.TextLine( "browser", new Point( xx, 3 * yyIncrement ) ) );

		ImageJPanel.TextLine textLineVersion = new ImageJPanel.TextLine( ApplicationConfiguration.VERSION, new Point( xx, yyIncrement * 4 ) );
		font = new Font("Calibri", Font.BOLD, 25);
		textLineVersion.setFont(font);
		textLineVersion.setColor( Color.GREEN.darker() );
		addTextLine( textLineVersion );

//		setForegroundColorForTextLines( new Color( 112, 173, 71 ) );	// Green
		setForegroundColorForTextLines( Color.BLACK );
	}

	protected void setWindowConfiguration( )
	{
		Vector<JPopupMenu> vectorJpopupMenus = null;

		MapResizeRelocateComponentItem mapRRCI = null;
/*
		a_intern = new JFrameInternationalization(	ApplicationConfiguration.sa_MAIN_FOLDER,
													ApplicationConfiguration.sa_APPLICATION_NAME,
													ApplicationConfiguration.sa_CONFIGURATION_GROUP,
													ApplicationConfiguration.sa_PROPERTIES_PATH_IN_JAR,
													a_configurationBaseFileName,
													this,
													parent,
													a_vectorJpopupMenus,
													false,
													mapRRCI );
*/
		createInternationalization(	getAppliConf().getConfigurationMainFolder(),
									getAppliConf().getApplicationNameFolder(),
									getAppliConf().getApplicationGroup(),
									getAppliConf().getInternationalPropertiesPathInJar(),
									sa_configurationBaseFileName,
									this,
									null,
									vectorJpopupMenus,
									true,
									mapRRCI );

		this.registerInternationalString(CONF_OPENING_CONFIGURATION, "Opening configuration ..." );
		this.registerInternationalString(CONF_SHOWING_LICENSES,	"Showing licenses ..." );
		this.registerInternationalString(CONF_PROBLEM_LAUNCHING_THE_APPLICATION, "Problem launching the application" );
		this.registerInternationalString(CONF_CREATING_MAIN_WINDOW, "Creating main window ..." );
		this.registerInternationalString(CONF_IMPORTING_CONFIGURATION, "Importing configuration ..." );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        getContentPane().setLayout(null);

        pack();
    }// </editor-fold>//GEN-END:initComponents

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
		try {
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(Splash.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(Splash.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(Splash.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(Splash.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
        //</editor-fold>
        //</editor-fold>

		/* Create and display the form */
/*
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				new Splash().setVisible(true);
			}
		});
*/
	}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables


	public static void free()
	{
		if( _instance != null )
		{
			_instance.setVisible( false );
			_instance.dispose();
			_instance = null;
		}
	}
}
