/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.chesspdfbrowser.threads;

import com.frojasg1.chesspdfbrowser.io.file.implementation.PgnChessFile;
import com.frojasg1.chesspdfbrowser.model.chess.game.ChessGame;
import java.util.List;

/**
 *
 * @author Usuario
 */
public class SavePGNThread extends Thread
{
	protected LoadControllerInterface _controller = null;
	protected String _fileName = null;
	protected List<ChessGame> _list = null;
	protected PgnChessFile _pgnFile = null;

	public SavePGNThread( LoadControllerInterface controller, PgnChessFile pgnFile, List<ChessGame> list, String fileName )
	{
		_pgnFile = pgnFile;
		_list = list;
		_controller = controller;
		_fileName = fileName;
	}

	@Override
	public void run()
	{
		try
		{
			if( _controller != null )	_controller.block();

			_pgnFile.saveToFile( _list, _fileName, null );

			if( _controller != null )
				_controller.newChessListGameLoaded( _list, _pgnFile );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			if( _controller != null )
			{
				_controller.showError( "Error saving PGN file " + _fileName + ". ERROR: " + th.getMessage(), "Error saving file" );
			}
		}
		finally
		{
			if( _controller != null )
				_controller.unblock();
		}
	}
}
