/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.chesspdfbrowser.view.chess.editcomment;

import com.frojasg1.chesspdfbrowser.io.notation.ChessMoveAlgebraicNotation;
import com.frojasg1.chesspdfbrowser.model.chess.game.ChessGame;
import com.frojasg1.chesspdfbrowser.model.chess.game.tree.MoveTreeNode;
import com.frojasg1.chesspdfbrowser.model.chess.game.tree.NAG;
import com.frojasg1.chesspdfbrowser.pdf.textsegmentation.implementation.Range;
import com.frojasg1.general.desktop.view.panels.AcceptCancelRevertControllerInterface;
import com.frojasg1.general.desktop.view.panels.AcceptCancelRevertPanel;
import com.frojasg1.chesspdfbrowser.view.chess.interaction.ChessGameControllerInterface;
import com.hotmail.frojasg1.applications.common.components.internationalization.JFrameInternationalization;
import com.hotmail.frojasg1.applications.common.components.internationalization.window.InternationalizedJFrame;
import com.hotmail.frojasg1.applications.common.components.resizecomp.MapResizeRelocateComponentItem;
import com.hotmail.frojasg1.applications.common.components.resizecomp.ResizeRelocateItem;
import com.frojasg1.general.StringFunctions;
import com.frojasg1.general.desktop.view.SelectionPreservingCaret;
import com.frojasg1.general.desktop.view.panels.InformerInterface;
import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JFrame;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

/**
 *
 * @author Usuario
 */
public class EditCommentFrame extends InternationalizedJFrame implements AcceptCancelRevertControllerInterface
{
	protected static final String CONF_EDIT_COMMENT = "EDIT_COMMENT";

	protected static final String sa_configurationBaseFileName = "EditCommentFrame";
	protected JFrame _parent = null;

	protected MoveTreeNode _moveTreeNode = null;
	protected ChessGame _chessGame = null;
	protected ChessGameControllerInterface _controller = null;

	protected List<NAG> _listOfNAGs = new ArrayList<NAG>();

	protected static final String BOLD_BLACK = "BOLD_BLACK";
	protected static final String GREEN = "GREEN";

	protected static EditCommentFrame _instance = null;

	AcceptCancelRevertPanel _acceptPanel = null;
	
	public static  EditCommentFrame createInstance( JFrame parent,
													EditCommentConfiguration applicationConfiguration,
													ChessGameControllerInterface controller )
	{
		if( _instance == null )
			_instance = new EditCommentFrame( parent, applicationConfiguration, controller );

		return( _instance );
	}

	public static EditCommentFrame instance()
	{
		return( _instance );
	}

	/**
	 * Creates new form EditCommentFrame
	 */
	protected EditCommentFrame( JFrame parent,
								EditCommentConfiguration applicationConfiguration,
								ChessGameControllerInterface controller )
	{
		super( applicationConfiguration );

		_parent = parent;
		_controller = controller;
		initComponents();

		initOwnComponents();

		addStyles();

		fillInComboBoxOfNAGs();

		updateAlwaysOnTopCheckBoxFromConfiguration();
		
		setWindowConfiguration();
	}

	protected void initOwnComponents()
	{
//		System.out.println( "EditCommentFrame" );
		_acceptPanel = new AcceptCancelRevertPanel( this );
//		_acceptPanel.setEraseButtonEnabled( false );
		_acceptPanel.setBounds( 0, 0, jPanel2.getWidth(), jPanel2.getHeight() );

		jPanel2.add( _acceptPanel );
	}

	protected void updateAlwaysOnTopCheckBoxFromConfiguration()
	{
		jCB_alwaysOnTop.setSelected( getAppliConf().getEditCommentWindowAlwaysOnTop() );
		updateAlwaysOnTop();
	}

	public void updateAlwaysOnTop()
	{
		setAlwaysOnTop( jCB_alwaysOnTop.isSelected() );
	}

	protected void addStyles()
	{
		jTP_listOfMoves.setCaret( new SelectionPreservingCaret() );

		StyledDocument sd = jTP_listOfMoves.getStyledDocument();

		Style defaultStyle = sd.getStyle(StyleContext.DEFAULT_STYLE);

		final Style bold = sd.addStyle(BOLD_BLACK, defaultStyle);
		StyleConstants.setFontSize(bold, StyleConstants.getFontSize( defaultStyle ) );
		StyleConstants.setForeground(bold, Color.BLACK );
		StyleConstants.setBold(bold, true);

/*
		sd = jTP_comment.getStyledDocument();

		defaultStyle = sd.getStyle(StyleContext.DEFAULT_STYLE);

		final Style green = sd.addStyle(GREEN, defaultStyle);
		StyleConstants.setFontSize(green, StyleConstants.getFontSize( defaultStyle ) );
		StyleConstants.setForeground(green, Color.GREEN.darker() );
		StyleConstants.setBold(green, false);
*/
//		jTP_comment.setForeground( Color.RED );
	}

	protected void fillInComboBoxOfNAGs()
	{
		Vector<NAG> listOfAvailableNAGs = new Vector<NAG>();

		NAG selectedNAG = new NAG(1);

		for( int ii=0; ii<256; ii++ )
		{
			if( ii != 1 )
				listOfAvailableNAGs.add( new NAG( ii ) );
			else
				listOfAvailableNAGs.add( selectedNAG );
		}

		DefaultComboBoxModel<NAG> dcbm = new DefaultComboBoxModel<NAG>( listOfAvailableNAGs );
		jCB_NAGs.setModel( dcbm );
		jCB_NAGs.setSelectedItem( selectedNAG );
	}

	protected void setWindowConfiguration( )
	{
		MapResizeRelocateComponentItem mapRRCI = new MapResizeRelocateComponentItem( );
		try
		{
			mapRRCI.putResizeRelocateComponentItem( jPanel1, ResizeRelocateItem.RESIZE_TO_RIGHT +
																ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( jScrollPane1, ResizeRelocateItem.RESIZE_TO_RIGHT );
			mapRRCI.putResizeRelocateComponentItem( jScrollPane2, ResizeRelocateItem.RESIZE_TO_RIGHT );
//			mapRRCI.putResizeRelocateComponentItem( jP_NAG, ResizeRelocateItem.RESIZE_TO_RIGHT );
//			mapRRCI.putResizeRelocateComponentItem( jLabel1, ResizeRelocateItem.RESIZE_TO_RIGHT );
//			mapRRCI.putResizeRelocateComponentItem( jLabel2, ResizeRelocateItem.RESIZE_TO_RIGHT );

//			HintForComponent hfc = new HintForComponent( jB_hide, "Esconder el formulario ..." );
			
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
/*
		a_intern = new JFrameInternationalization(	ApplicationConfiguration.sa_MAIN_FOLDER,
													ApplicationConfiguration.sa_APPLICATION_NAME,
													ApplicationConfiguration.sa_CONFIGURATION_GROUP,
													ApplicationConfiguration.sa_PATH_PROPERTIES_IN_JAR,
													a_configurationBaseFileName,
													this,
													parent,
													a_vectorJpopupMenus,
													false,
													mapRRCI );
*/
		createInternationalization(	getAppliConf().getConfigurationMainFolder(),
									getAppliConf().getApplicationNameFolder(),
									getAppliConf().getApplicationGroup(),
									getAppliConf().getInternationalPropertiesPathInJar(),
									sa_configurationBaseFileName,
									this,
									_parent,
									null,
									true,
									mapRRCI );

		registerInternationalString(CONF_EDIT_COMMENT, "Edit comment" );
		a_intern.setMaxWindowHeightNoLimit( false );

//		setIcon( "com/frojasg1/chesspdfbrowser/resources/icons/App.icon.png" );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jL_move = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTP_listOfMoves = new javax.swing.JTextPane();
        jL_comment = new javax.swing.JLabel();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTP_comment = new javax.swing.JTextPane();
        jP_NAG = new javax.swing.JPanel();
        jB_addNAG = new javax.swing.JButton();
        jB_clearNAGs = new javax.swing.JButton();
        jCB_NAGs = new javax.swing.JComboBox();
        jL_pgnNAGs = new javax.swing.JLabel();
        jL_NAGstring = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jCB_alwaysOnTop = new javax.swing.JCheckBox();
        jPanel2 = new javax.swing.JPanel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setMaximumSize(new java.awt.Dimension(2147483647, 406));
        setMinimumSize(new java.awt.Dimension(476, 406));
        setName(""); // NOI18N
        setPreferredSize(new java.awt.Dimension(476, 406));
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });
        getContentPane().setLayout(null);

        jPanel1.setMinimumSize(new java.awt.Dimension(460, 10));
        jPanel1.setName("jPanel1"); // NOI18N
        jPanel1.setPreferredSize(new java.awt.Dimension(460, 381));
        jPanel1.setLayout(null);

        jL_move.setText("Move :");
        jL_move.setName("jL_move"); // NOI18N
        jPanel1.add(jL_move);
        jL_move.setBounds(10, 20, 160, 14);

        jTP_listOfMoves.setEditable(false);
        jScrollPane1.setViewportView(jTP_listOfMoves);

        jPanel1.add(jScrollPane1);
        jScrollPane1.setBounds(0, 40, 460, 70);

        jL_comment.setText("Comment :");
        jL_comment.setName("jL_comment"); // NOI18N
        jPanel1.add(jL_comment);
        jL_comment.setBounds(10, 120, 180, 14);

        jTP_comment.setForeground(new java.awt.Color(0, 178, 0));
        jScrollPane2.setViewportView(jTP_comment);

        jPanel1.add(jScrollPane2);
        jScrollPane2.setBounds(0, 140, 460, 70);

        jP_NAG.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jP_NAG.setName("jP_NAG"); // NOI18N
        jP_NAG.setLayout(null);

        jB_addNAG.setText("Add NAG");
        jB_addNAG.setName("jB_addNAG"); // NOI18N
        jB_addNAG.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_addNAGActionPerformed(evt);
            }
        });
        jP_NAG.add(jB_addNAG);
        jB_addNAG.setBounds(10, 50, 120, 23);

        jB_clearNAGs.setText("Clear NAGs");
        jB_clearNAGs.setName("jB_clearNAGs"); // NOI18N
        jB_clearNAGs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_clearNAGsActionPerformed(evt);
            }
        });
        jP_NAG.add(jB_clearNAGs);
        jB_clearNAGs.setBounds(10, 10, 120, 23);

        jP_NAG.add(jCB_NAGs);
        jCB_NAGs.setBounds(150, 50, 150, 20);

        jL_pgnNAGs.setText("PGN NAGs :");
        jL_pgnNAGs.setName("jL_pgnNAGs"); // NOI18N
        jP_NAG.add(jL_pgnNAGs);
        jL_pgnNAGs.setBounds(150, 30, 100, 14);

        jL_NAGstring.setText("NAG string :");
        jL_NAGstring.setName("jL_NAGstring"); // NOI18N
        jP_NAG.add(jL_NAGstring);
        jL_NAGstring.setBounds(150, 10, 100, 14);

        jLabel1.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jLabel1.setMaximumSize(new java.awt.Dimension(2000, 2000));
        jLabel1.setPreferredSize(new java.awt.Dimension(16, 220));
        jP_NAG.add(jLabel1);
        jLabel1.setBounds(250, 10, 180, 16);

        jLabel2.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jLabel2.setMaximumSize(new java.awt.Dimension(2000, 2000));
        jLabel2.setPreferredSize(new java.awt.Dimension(16, 220));
        jP_NAG.add(jLabel2);
        jLabel2.setBounds(250, 30, 180, 16);

        jPanel1.add(jP_NAG);
        jP_NAG.setBounds(10, 220, 440, 80);

        jCB_alwaysOnTop.setText("Always on top");
        jCB_alwaysOnTop.setName("jCB_alwaysOnTop"); // NOI18N
        jCB_alwaysOnTop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCB_alwaysOnTopActionPerformed(evt);
            }
        });
        jPanel1.add(jCB_alwaysOnTop);
        jCB_alwaysOnTop.setBounds(260, 0, 200, 23);

        jPanel2.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 102, 204)));
        jPanel2.setMinimumSize(new java.awt.Dimension(130, 50));
        jPanel2.setPreferredSize(new java.awt.Dimension(130, 50));
        jPanel2.setLayout(null);
        jPanel1.add(jPanel2);
        jPanel2.setBounds(160, 310, 130, 50);

        getContentPane().add(jPanel1);
        jPanel1.setBounds(0, 0, 460, 370);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jB_clearNAGsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_clearNAGsActionPerformed
        // TODO add your handling code here:

		_listOfNAGs.clear();
		updateNAGStrings();

    }//GEN-LAST:event_jB_clearNAGsActionPerformed

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        // TODO add your handling code here:

		formWindowClosing();

    }//GEN-LAST:event_formWindowClosing

    private void jB_addNAGActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_addNAGActionPerformed
        // TODO add your handling code here:

		NAG selectedNAG = (NAG) jCB_NAGs.getSelectedItem();

		if( selectedNAG != null )
			_listOfNAGs.add(selectedNAG);

		updateNAGStrings();

    }//GEN-LAST:event_jB_addNAGActionPerformed

    private void jCB_alwaysOnTopActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCB_alwaysOnTopActionPerformed
        // TODO add your handling code here:

		setAlwaysOnTop( jCB_alwaysOnTop.isSelected() );

    }//GEN-LAST:event_jCB_alwaysOnTopActionPerformed

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
		try {
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(EditCommentFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(EditCommentFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(EditCommentFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(EditCommentFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
        //</editor-fold>

		/* Create and display the form */
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
//				new EditCommentFrame().setVisible(true);
			}
		});
	}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jB_addNAG;
    private javax.swing.JButton jB_clearNAGs;
    private javax.swing.JComboBox jCB_NAGs;
    private javax.swing.JCheckBox jCB_alwaysOnTop;
    private javax.swing.JLabel jL_NAGstring;
    private javax.swing.JLabel jL_comment;
    private javax.swing.JLabel jL_move;
    private javax.swing.JLabel jL_pgnNAGs;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JPanel jP_NAG;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JTextPane jTP_comment;
    private javax.swing.JTextPane jTP_listOfMoves;
    // End of variables declaration//GEN-END:variables

	public void setMove( ChessGame cg, MoveTreeNode mtn )
	{
		_chessGame = cg;

		if( _chessGame != null )
			setTitle( getInternationalString( CONF_EDIT_COMMENT ) +
						" - " + cg.getChessGameHeaderInfo().getDescriptionOfGame() );
		else
			setTitle( getInternationalString( CONF_EDIT_COMMENT ) );

		_moveTreeNode = mtn;

		rollback();
	}

	protected void rollback()
	{
		resetListOfNAGs();

		updateWindow();
	}

	protected void commit()
	{
		if( _moveTreeNode.getMove() != null )
		{
			updateMoveTreeNode();
			_controller.newChessGameChosen( _chessGame, true );
		}
	}

	protected void resetListOfNAGs()
	{
		_listOfNAGs.clear();

		if( ( _moveTreeNode != null ) && ( _moveTreeNode.getMove() != null ) )
		{
			Iterator<NAG> it = _moveTreeNode.getMove().getNagIterator();
			if( _moveTreeNode.getMove().getMoveToken() != null )
				it = _moveTreeNode.getMove().getMoveToken().nagIterator();

			while( it.hasNext() )
				_listOfNAGs.add( it.next() );
		}
	}

	protected String getNAGStringsToShow( List<NAG> list )
	{
		StringBuilder sb = new StringBuilder();

		String separator = "";
		Iterator<NAG> it = list.iterator();
		while( it.hasNext() )
		{
			sb.append(separator);
			sb.append( it.next().getStringToShow() );
			separator = " ";
		}

		return( sb.toString() );
	}

	protected String getPgnNAGs( List<NAG> list )
	{
		StringBuilder sb = new StringBuilder();

		String separator = "";
		Iterator<NAG> it = list.iterator();
		while( it.hasNext() )
		{
			sb.append(separator);
			sb.append( it.next().getStringForPGNfile() );
			separator = " ";
		}

		return( sb.toString() );
	}

	protected void updateWindow()
	{
		String listOfMovesString = "";
		String comment = "";
		if( ( _chessGame != null ) && ( _moveTreeNode != null ) )
		{
			try
			{
				listOfMovesString = _chessGame.getMoveListString( _moveTreeNode );
			}
			catch( Throwable th )
			{}

			comment = _moveTreeNode.getComment();
		}

		setListOfMoves( listOfMovesString );
		setComment( comment );

		updateNAGStrings();
	}

	protected void updateNAGStrings( )
	{
		jLabel1.setText(getNAGStringsToShow( _listOfNAGs ) );
		jLabel2.setText( getPgnNAGs( _listOfNAGs ) );
	}

	protected Range getLastMoveRange( String listOfMoves )
	{
		Range result = new Range(0,0);

		if( ( listOfMoves != null ) &&
			( listOfMoves.length() > 2 ) )
		{
			int end = listOfMoves.length() - 1;
			boolean found = false;

			while( ( end > -1 ) && !found )
			{
				int start = StringFunctions.lastIndexOfAnyChar(listOfMoves, " .", end ) + 1;
				if( start < 0 )
					start = 0;

				if( start <= end )
				{
					found = isAMove( listOfMoves.substring( start, end + 1 ) );

					if( found )
						result = new Range( start, end );
				}

				end = start - 2;
			}
		}

		return( result );
	}

	protected void setListOfMoves( String listOfMoves )
	{
		Range rangeOfLastMove = getLastMoveRange( listOfMoves );

		Style style = jTP_listOfMoves.getStyle( BOLD_BLACK );
		Style defaultStyle = jTP_listOfMoves.getStyle( StyleContext.DEFAULT_STYLE );

		jTP_listOfMoves.setText( listOfMoves );
		jTP_listOfMoves.getStyledDocument().setCharacterAttributes( 0, listOfMoves.length(),
													defaultStyle, true);
		jTP_listOfMoves.getStyledDocument().setCharacterAttributes( rangeOfLastMove.getInitial(),
																	rangeOfLastMove.getFinal() - rangeOfLastMove.getInitial() + 1,
																	style, true);

		Component focusOwner = JFrameInternationalization.getFocusedComponent();
		jTP_listOfMoves.requestFocus();

		jTP_listOfMoves.setSelectionStart( rangeOfLastMove.getInitial() );
		jTP_listOfMoves.setSelectionEnd( rangeOfLastMove.getFinal() + 1 );
		jTP_listOfMoves.repaint();

		if( focusOwner != null )
			focusOwner.requestFocus();
	}

	protected void setComment( String comment )
	{
		if( comment == null )
			comment = "";

//		Style style = jTP_comment.getStyle( GREEN );

		jTP_comment.setText( comment );
//		jTP_comment.getStyledDocument().setCharacterAttributes( 0, comment.length(), style, true);
	}

	protected void updateMoveTreeNode()
	{
		if( _moveTreeNode != null )
		{
			_moveTreeNode.setComment( jTP_comment.getText() );

			_moveTreeNode.getMove().getMoveToken().clearNAGlist();
			_moveTreeNode.getMove().getMoveToken().addAllNAGs( _listOfNAGs.iterator() );
		}
	}
	
	@Override
	public EditCommentConfiguration getAppliConf()
	{
		return( (EditCommentConfiguration) super.getAppliConf() );
	}

	@Override
	public void formWindowClosing( boolean closeWindow )
	{
		getAppliConf().setEditCommentWindowAlwaysOnTop( jCB_alwaysOnTop.isSelected() );

//		super.formWindowClosing( closeWindow );
		super.formWindowClosing( false );
		if( closeWindow )
			setVisible( false );
	}

	protected boolean isAMove( String text )
	{
		boolean result = false;

		if( _chessGame != null )
		{
			String translatedText = null;
			
			translatedText = _chessGame.getChessViewConfiguration().getChessLanguageConfigurationToShow().translateMoveStringToEnglish(text);
			result = ChessMoveAlgebraicNotation.getInstance().isItAChessMoveString(translatedText);
		}

		return( result );
	}

	protected void formWindowClosing()
	{
		setVisible( false );
		boolean closeWindow = false;

		formWindowClosing( closeWindow );
	}

	@Override
	public void accept(InformerInterface panel)
	{
		commit();
	}

	@Override
	public void cancel(InformerInterface panel)
	{
		formWindowClosing();
	}

	@Override
	public void revert(InformerInterface panel )
	{
		rollback();
	}

}
