/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.chesspdfbrowser.view.chess.images;

import com.frojasg1.chesspdfbrowser.model.chess.pieces.Bishop;
import com.frojasg1.chesspdfbrowser.model.chess.pieces.ChessPiece;
import com.frojasg1.chesspdfbrowser.model.chess.pieces.King;
import com.frojasg1.chesspdfbrowser.model.chess.pieces.Knight;
import com.frojasg1.chesspdfbrowser.model.chess.pieces.Pawn;
import com.frojasg1.chesspdfbrowser.model.chess.pieces.Queen;
import com.frojasg1.chesspdfbrowser.model.chess.pieces.Rook;
import com.frojasg1.general.desktop.ImageFunctions;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.imageio.ImageIO;

/**
 *
 * @author Usuario
 */
public class ChessBoardImages
{
	protected static final String _resourcePath = "com/frojasg1/chesspdfbrowser/resources/chess/images";

	protected static final int TRANSPARENT_COLOR = 0xFFB0B0B0;
	
	protected Map< String, BufferedImage > _originalPiecesMap;
	protected Map< Integer, Map< String, BufferedImage > > _resizedPiecesMap;
	protected Map< Integer, Map< String, BufferedImage > > _resizedSemiTransparentPiecesMap;

	protected BufferedImage[] _originalSquaresArray;
	protected Map< Integer, BufferedImage[] > _resizedSquaresMap;

	protected int _currentSquareWidth = -1;
	protected int _currentPieceWidth = -1;
	
	protected static ChessBoardImages _instance = null;

	public static ChessBoardImages instance()
	{
		if( _instance == null )
		{
			try
			{
				_instance = new ChessBoardImages();
			}
			catch( Throwable th )
			{
				th.printStackTrace();
				throw( new RuntimeException( "Error loading the images" ) );
			}
		}

		return( _instance );
	}
	
	protected ChessBoardImages() throws IOException
	{
		initialize();
	}
	
	protected String getResourceName( String shortResourceName )
	{
		return( _resourcePath + "/" + shortResourceName );
	}

	public void initialize() throws IOException
	{
		_originalPiecesMap = new HashMap< String, BufferedImage >();
		_originalSquaresArray = new BufferedImage[2];
		
//		int transparentColor = 0xFFB0B0B0;
		Integer transparentColor = null;
		
		insertNewOriginalPiece( ChessPiece.getPieceCode(Pawn.PIECE_TYPE_CODE, ChessPiece.WHITE ), getResourceName( "processed.white.pawn.png" ), transparentColor );
		insertNewOriginalPiece( ChessPiece.getPieceCode(Knight.PIECE_TYPE_CODE, ChessPiece.WHITE ), getResourceName( "processed.white.knight.png" ), transparentColor );
		insertNewOriginalPiece( ChessPiece.getPieceCode(Bishop.PIECE_TYPE_CODE, ChessPiece.WHITE ), getResourceName( "processed.white.bishop.png" ), transparentColor );
		insertNewOriginalPiece( ChessPiece.getPieceCode(Rook.PIECE_TYPE_CODE, ChessPiece.WHITE ), getResourceName( "processed.white.rook.png" ), transparentColor );
		insertNewOriginalPiece( ChessPiece.getPieceCode(Queen.PIECE_TYPE_CODE, ChessPiece.WHITE ), getResourceName( "processed.white.queen.png" ), transparentColor );
		insertNewOriginalPiece( ChessPiece.getPieceCode(King.PIECE_TYPE_CODE, ChessPiece.WHITE ), getResourceName( "processed.white.king.png" ), transparentColor );

		insertNewOriginalPiece( ChessPiece.getPieceCode(Pawn.PIECE_TYPE_CODE, ChessPiece.BLACK ), getResourceName( "processed.black.pawn.png" ), transparentColor );
		insertNewOriginalPiece( ChessPiece.getPieceCode(Knight.PIECE_TYPE_CODE, ChessPiece.BLACK ), getResourceName( "processed.black.knight.png" ), transparentColor );
		insertNewOriginalPiece( ChessPiece.getPieceCode(Bishop.PIECE_TYPE_CODE, ChessPiece.BLACK ), getResourceName( "processed.black.bishop.png" ), transparentColor );
		insertNewOriginalPiece( ChessPiece.getPieceCode(Rook.PIECE_TYPE_CODE, ChessPiece.BLACK ), getResourceName( "processed.black.rook.png" ), transparentColor );
		insertNewOriginalPiece( ChessPiece.getPieceCode(Queen.PIECE_TYPE_CODE, ChessPiece.BLACK ), getResourceName( "processed.black.queen.png" ), transparentColor );
		insertNewOriginalPiece( ChessPiece.getPieceCode(King.PIECE_TYPE_CODE, ChessPiece.BLACK ), getResourceName( "processed.black.king.png" ), transparentColor );

		_originalSquaresArray[0] = getResourceImage( getResourceName( "BlackSquare.png" ) );
		_originalSquaresArray[1] = getResourceImage( getResourceName( "WhiteSquare.png" ) );

		_resizedSquaresMap = new HashMap< Integer, BufferedImage[] >();
		_resizedPiecesMap = new HashMap< Integer, Map< String, BufferedImage > >();
		_resizedSemiTransparentPiecesMap = new HashMap< Integer, Map< String, BufferedImage > >();

		resizeSquares( _originalSquaresArray[0].getWidth() );
	}

	public static BufferedImage getResourceImage( String resourceImageName ) throws IOException
	{
		BufferedImage result = ImageIO.read( ClassLoader.getSystemClassLoader().getResourceAsStream( resourceImageName ) );
		
		return( result );
	}
	
	protected void insertNewOriginalPiece( String pieceCode, String resourceImageName, Integer transparentColor ) throws IOException
	{
		BufferedImage bi = getResourceImage( resourceImageName );
		
		BufferedImage imageWithTransparentColor = ImageFunctions.resizeImage( bi, bi.getWidth(), bi.getHeight(), transparentColor, null, null );
		
		_originalPiecesMap.put( pieceCode, imageWithTransparentColor );
	}
/*
	protected void resizeSquaresAndPieces( int newSquareWidth, int newPieceWidth )
	{
		if( newSquareWidth != _currentSquareWidth )
		{
			resizeSquares( newSquareWidth );
			_currentSquareWidth = newSquareWidth;
		}
		
		if( newPieceWidth != _currentPieceWidth )
		{
			_resizedPiecesMap = resizePieces( newPieceWidth, 0 );
			_resizedSemiTransparentPiecesMap =  resizePieces( newPieceWidth, 200 );
			_currentPieceWidth = newPieceWidth;
		}
	}
*/
	protected BufferedImage[] resizeSquares( int newSquareWidth )
	{
		BufferedImage[] result = new BufferedImage[2];
		result[ ChessPiece.WHITE ] = ImageFunctions.resizeSquaredImage(_originalSquaresArray[ChessPiece.WHITE],
																		newSquareWidth, null, null, 255 );
		result[ ChessPiece.BLACK ] = ImageFunctions.resizeSquaredImage(_originalSquaresArray[ChessPiece.BLACK],
																		newSquareWidth, null, null, 255 );
		return( result );
	}

	protected Map<String, BufferedImage> resizePieces( int newPieceWidth, Integer alpha )
	{		
		Map<String, BufferedImage> newMap = new HashMap<String, BufferedImage>();

		Iterator< Map.Entry<String, BufferedImage> > it = _originalPiecesMap.entrySet().iterator();
		
		while( it.hasNext() )
		{
			Map.Entry<String, BufferedImage> pair = it.next();
			
			String key = pair.getKey();
			BufferedImage originalImage = pair.getValue();
			
			BufferedImage transformedImage = ImageFunctions.resizeSquaredImage( originalImage, newPieceWidth, TRANSPARENT_COLOR, 0x00F8E2E2, alpha );
			
			newMap.put( key, transformedImage );
		}

		return( newMap );
	}

	protected BufferedImage resizePiece( String pieceCode, int newPieceWidth, Integer alpha )
	{		
		BufferedImage originalImage = _originalPiecesMap.get( pieceCode );
			
		BufferedImage transformedImage = ImageFunctions.resizeSquaredImage( originalImage, newPieceWidth, TRANSPARENT_COLOR, 0x00F8E2E2, alpha );
		return( transformedImage );
	}

	/**
	 * It returns the transformed image for an Square.
	 * 
	 * @param color     0: BLACK, 1: WHITE
	 * @return 
	 */
	public BufferedImage getSquareImage( int color, int squareWidth )
	{
//		System.out.println( "_resizedSquaresMap is " + ( _resizedSquaresMap != null ? "NOT" : "" ) + " NULL.   color=" + color );
		BufferedImage bi = null;

		if( squareWidth > 0 )
		{
			BufferedImage[] tmpArray = _resizedSquaresMap.get( squareWidth );

			if( tmpArray == null )
			{
				tmpArray = resizeSquares( squareWidth );
				_resizedSquaresMap.put( squareWidth, tmpArray );
			}

			bi = tmpArray[ color ];
		}

		return( bi );
	}

	public BufferedImage getPieceImage( String pieceCode, int pieceWidth )
	{
		BufferedImage result = null;
		if( ( pieceCode != null ) && ( pieceWidth > 0 ) )
		{
			if( (pieceCode.length()==3) && (pieceCode.charAt(0)=='O' ) )
				result = _originalPiecesMap.get( pieceCode.substring( 1, 3 ) );
			else
			{
				Map< String, BufferedImage > tmpMap = _resizedPiecesMap.get( pieceWidth );

				if( tmpMap == null )
				{
					int alpha = 255;
					tmpMap = resizePieces( pieceWidth, alpha );
					_resizedPiecesMap.put( pieceWidth, tmpMap );
				}
				result = tmpMap.get( pieceCode );
			}
		}
		return( result );
	}

	public BufferedImage getSemiTransparentPieceImage( String pieceCode, int pieceWidth )
	{
		BufferedImage result = null;
		if( ( pieceCode != null ) && ( pieceWidth > 0 ) )
		{
			Map< String, BufferedImage > tmpMap = _resizedSemiTransparentPiecesMap.get( pieceWidth );

			if( tmpMap == null )
			{
				int alpha = 60;
				tmpMap = resizePieces( pieceWidth, alpha );
				_resizedSemiTransparentPiecesMap.put( pieceWidth, tmpMap );
			}
			result = tmpMap.get( pieceCode );
		}
		return( result );
	}

	public static void main( String[] args )
	{
		try
		{
			ChessBoardImages cbi = new ChessBoardImages();
//			cbi.resizeSquaresAndPieces(240, 135);
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
	}

}
